@@ -1,8 +1,8 @@
 
-__version__ = "0.12.6"
+__version__ = "0.12.7"
 __version_vcs_info__ = {
-    'branch': 'hotfix-0.12.6',
-    'revid': '5d7df89',
-    'revno': 2156}
+    'branch': 'master',
+    'revid': '5bd75cb',
+    'revno': 2160}
 __version_user_email__ = "saxtouri@admin.grnet.gr"
 __version_user_name__ = "Stavros Sachtouris"
